@@ -34,6 +34,8 @@ A good library to use for in-memory caching is cachetools (https://pypi.org/proj
 Configure using environment variables
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
+See `Configuration <conf.html>`_ for more configuration options.
+
 Especially when using the `vaex server <server.html>`_ it can be useful to turn on caching externally using enviroment variables.
 
     $ VAEX_CACHE=disk VAEX_CACHE_DISK_SIZE_LIMIT="10GB" python -m vaex.server
@@ -62,14 +64,13 @@ from filelock import FileLock
 from vaex.promise import Promise
 
 import vaex.utils
+import vaex.settings
+
 diskcache = vaex.utils.optional_import('diskcache')
 _redis = vaex.utils.optional_import('redis')
 cachetools = vaex.utils.optional_import('cachetools')
 
 log = logging.getLogger('vaex.cache')
-_cache_tasks_type = vaex.utils.get_env_type(str, 'VAEX_CACHE', None)  # disk/redis/memory_infinite
-disk_size_limit = vaex.utils.get_env_type(str, 'VAEX_CACHE_DISK_SIZE_LIMIT', '1GB')
-memory_size_limit = vaex.utils.get_env_type(str, 'VAEX_CACHE_MEMORY_SIZE_LIMIT', '1GB')
 
 cache = None
 # used for testing
@@ -169,7 +170,7 @@ def memory_infinite(clear=False):
 
 
 @_with_cleanup
-def memory(maxsize=memory_size_limit, classname="LRUCache", clear=False):
+def memory(maxsize=vaex.settings.cache.memory_size_limit, classname="LRUCache", clear=False):
     """Sets a memory cache using cachetools (https://cachetools.readthedocs.io/).
 
     Calling multiple times with clear=False will keep the current cache (useful in notebook usage).
@@ -196,10 +197,10 @@ def memory(maxsize=memory_size_limit, classname="LRUCache", clear=False):
 
 
 @_with_cleanup
-def disk(clear=False, size_limit=disk_size_limit, eviction_policy="least-recently-stored"):
+def disk(clear=False, size_limit=vaex.settings.cache.disk_size_limit, eviction_policy="least-recently-stored"):
     """Stored cached values using the diskcache library.
 
-    The path to store the cache is: ~/.vaex/cache/diskcache
+    See configuration details at `configuration of cache <conf.html#disk-size-limit>`_. and `configuration of paths <conf.html#cache-compute>`_
 
     :param int or str size_limit: Max size of cache in bytes (or use a string like '128MB')
         See http://www.grantjenks.com/docs/diskcache/tutorial.html?highlight=eviction#tutorial-settings for more details.
@@ -211,7 +212,7 @@ def disk(clear=False, size_limit=disk_size_limit, eviction_policy="least-recentl
     size_limit = parse_bytes(size_limit)
     global cache
     old_cache = cache
-    path = vaex.utils.get_private_dir('cache/diskcache')
+    path = vaex.settings.cache.path
     if clear:
         try:
             log.debug(f"Clearing disk cache: {path}")
@@ -489,5 +490,5 @@ def _memoize(f=None, key_function=None, type='computed', delay=False):
         return wrapper_top()
 
 
-if _cache_tasks_type:
-    on(_cache_tasks_type)
+if vaex.settings.cache.type:
+    on(vaex.settings.cache.type)
