@@ -8,11 +8,12 @@ import numpy as np
 import pyarrow as pa
 
 import vaex
+import vaex.utils
 import vaex.cache
 from .array_types import supported_array_types, supported_arrow_array_types, string_types, is_string_type
 
-on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
-if not on_rtd:
+
+if vaex.utils.has_c_extension:
     import vaex.strings
 
 logger = logging.getLogger("vaex.column")
@@ -62,12 +63,56 @@ class ColumnVirtualRange(Column):
 
     def __getitem__(self,  slice):
         start, stop, step = slice.start, slice.stop, slice.step
+        if start is None:
+            start = self.start
+        if stop is None:
+            stop = self.stop
+        if step is None:
+            step = self.step
         return np.arange(self.start + start, self.start + stop, step, dtype=self.dtype)
 
     def trim(self, i1, i2):
         return ColumnVirtualRange(self.start + i1 * self.step, self.start + i2 * self.step, self.step, self.dtype)
 
 
+class ColumnVirtualConstant(Column):
+    def __init__(self, value, length, dtype=None, chunk_size=1024):
+        self.value = value
+        self.length = length
+        self.dtype = self._get_dtype(dtype)
+        self.chunk_size = chunk_size
+
+    def __len__(self):
+        return self.length
+
+    def __getitem__(self, item):
+        template = pa.array([self.value] * self.chunk_size, type=self.dtype.arrow)
+        n_chunks = (self.length + self.chunk_size - 1) // self.chunk_size
+        if item.step not in [None, 1]:
+            raise ValueError('step can only be 1')
+        start, stop, step = item.start, item.stop, item.step
+        start = start or 0
+        stop = stop or len(self)
+        assert step in [None, 1]
+        item = slice(start, stop, step)
+        ar = pa.chunked_array([template] * n_chunks)[item]
+        return ar
+
+    def _fingerprint(self):
+        fp = vaex.cache.fingerprint(self.value, self.length, self.dtype, self.chunk_size)
+        return f'column-vconstant-{fp}'
+
+    def _get_dtype(self, dtype):
+        if dtype is None:
+            return vaex.datatype.DataType(pa.array([self.value]).type)
+        else:
+            return vaex.datatype.DataType(dtype)
+
+    def trim(self, i1, i2):
+        length = i2 - i1
+        return ColumnVirtualConstant(value=self.value, length=length)
+
+
 class ColumnMaskedNumpy(Column):
     def __init__(self, data, mask):
         self.data = data
