@@ -2,6 +2,7 @@
 
 from functools import reduce
 import logging
+import operator
 import sys
 
 import numpy as np
@@ -60,6 +61,9 @@ class TaskPart:
     def ideal_splits(self, nthreads):
         return nthreads
 
+    def memory_usage(self):
+        return 0
+
 
 @register
 class TaskPartSum(TaskPart):
@@ -69,6 +73,9 @@ class TaskPartSum(TaskPart):
         self.total = 0
         self.expression = expression
 
+    def get_bin_count(self):
+        return 1
+
     @property
     def expressions(self):
         return [self.expression]
@@ -76,8 +83,8 @@ class TaskPartSum(TaskPart):
     def get_result(self):
         return self.total
 
-    def process(self, thread_index, i1, i2, filter_mask, chunk):
-        self.total += chunk.sum()
+    def process(self, thread_index, i1, i2, filter_mask, selection_masks, blocks):
+        self.total += blocks[0].sum()
 
     def reduce(self, others):
         self.total += sum(other.total for other in others)
@@ -95,6 +102,9 @@ class TaskPartFilterFill(TaskPart):
     def __init__(self):
         pass
 
+    def get_bin_count(self):
+        return 0
+
     @property
     def expressions(self):
         return []
@@ -102,7 +112,7 @@ class TaskPartFilterFill(TaskPart):
     def get_result(self):
         return None
 
-    def process(self, thread_index, i1, i2, filter_mask):
+    def process(self, thread_index, i1, i2, filter_mask, selection_masks, blocks):
         assert filter_mask is not None, f'{filter_mask}'
 
     def reduce(self, others):
@@ -148,6 +158,9 @@ class TaskPartSetCreate(TaskPart):
         # *7 is arbitrary, but we can have more maps than threads to avoid locks
         self.set = self.ordered_set_type(self.nthreads*7)
 
+    def get_bin_count(self):
+        return len(self.set)
+
     @property
     def expressions(self):
         return [self.expression]
@@ -158,11 +171,12 @@ class TaskPartSetCreate(TaskPart):
         else:
             return self.set
 
-    def process(self, thread_index, i1, i2, filter_mask, ar):
+    def process(self, thread_index, i1, i2, filter_mask, selection_masks, blocks):
+        ar = blocks[0]
         from vaex.column import _to_string_sequence
         self._check_row_limit()
         if self.selection:
-            selection_mask = self.df.evaluate_selection_mask(self.selection, i1=i1, i2=i2, cache=True)
+            selection_mask = selection_masks[0]
             ar = filter(ar, selection_mask)
         if len(ar) == 0:
             return
@@ -172,7 +186,7 @@ class TaskPartSetCreate(TaskPart):
             ar = _to_string_sequence(ar)
         else:
             ar = vaex.array_types.to_numpy(ar)
-            if ar.strides != (1,):
+            if ar.strides != (ar.itemsize,):
                 ar = ar.copy()
         chunk_size = 1024*1024
 
@@ -228,14 +242,15 @@ class TaskPartSetCreate(TaskPart):
             if count > self.unique_limit:
                 raise vaex.RowLimitException(f'Resulting set has {count:,} unique combinations, which is larger than the allowed value of {self.unique_limit:,}')
         self.set = set_merged
-        fp = vaex.cache.fingerprint(self.expression, self.dtype, self.dtype_item, self.flatten, self.selection)
-        self.set.fingerprint = f'set-df-{self.df_fp}-{fp}'
+        self.set.fingerprint = f'set-{self.fingerprint}'
 
     @classmethod
     def decode(cls, encoding, spec, df, nthreads):
         return cls(df, spec['expression'], encoding.decode('dtype', spec['dtype']), encoding.decode('dtype', spec['dtype_item']),
                    flatten=spec['flatten'], unique_limit=spec['unique_limit'], selection=spec['selection'], return_inverse=spec['return_inverse'], nthreads=nthreads)
 
+    def memory_usage(self):
+        return sys.getsizeof(self.set)
 
 
 @register
@@ -260,7 +275,11 @@ class TaskPartMapReduce(TaskPart):
         self.selection = selection
         self.values = []
 
-    def process(self, thread_index, i1, i2, filter_mask, *blocks):
+    def get_bin_count(self):
+        return 0
+
+    def process(self, thread_index, i1, i2, filter_mask, selection_masks, blocks):
+        selection_mask = selection_masks[0]
         if self.to_numpy:
             blocks = [block if isinstance(block, np.ndarray) else block.to_numpy() for block in blocks]
         if self.to_float:
@@ -280,16 +299,17 @@ class TaskPartMapReduce(TaskPart):
             selection = self.selection
             if self.pre_filter:
                 if selection:
-                    selection_mask = self.df.evaluate_selection_mask(selection, i1=i1, i2=i2, cache=True)
                     blocks = [filter(block, selection_mask) for block in blocks]
             else:
-                if selection or self.df.filtered:
-                    selection_mask = self.df.evaluate_selection_mask(selection, i1=i1, i2=i2, cache=True, pre_filtered=False)
-                    if filter_mask is not None:
-                        selection_mask = selection_mask & filter_mask
+                if selection and self.df.filtered:
+                    selection_mask = selection_mask & filter_mask
+                    blocks = [filter(block, selection_mask) for block in blocks]
+                elif self.df.filtered:
+                    blocks = [filter(block, filter_mask) for block in blocks]
+                elif selection:
                     blocks = [filter(block, selection_mask) for block in blocks]
         if self.info:
-            self.values.append(self._map(thread_index, i1, i2, *blocks))
+            self.values.append(self._map(thread_index, i1, i2, selection_mask, blocks))
         else:
             self.values.append(self._map(*blocks))
 
@@ -333,7 +353,10 @@ class TaskPartStatistic(TaskPart):
         self.edges = edges
         self.selection_waslist = selection_waslist
 
-    def process(self, thread_index, i1, i2, filter_mask, *blocks):
+    def get_bin_count(self):
+        return reduce(operator.mul, self.shape, 1)
+
+    def process(self, thread_index, i1, i2, filter_mask, selection_masks, blocks):
         class Info(object):
             pass
         info = Info()
@@ -372,7 +395,7 @@ class TaskPartStatistic(TaskPart):
         this_thread_grid = self.grid
         for i, selection in enumerate(self.selections):
             if selection:
-                selection_mask = self.df.evaluate_selection_mask(selection, i1=i1, i2=i2, cache=True)  # TODO
+                selection_mask = selection_masks[i]
                 if selection_mask is None:
                     raise ValueError("performing operation on selection while no selection present")
                 if mask is not None:
@@ -454,6 +477,7 @@ class TaskPartAggregation(TaskPart):
         self.df = df
         self.has_values = False
         self.dtypes = dtypes
+        self.binners = binners
         # self.expressions_all = expressions
         self.expressions = [binner.expression for binner in binners]
         # TODO: selection and edges in descriptor?
@@ -484,6 +508,12 @@ class TaskPartAggregation(TaskPart):
             initial_values_i = initial_values[i] if initial_values else None
             self.aggregations.append((aggregator_descriptor, selections, list(create_aggregator(aggregator_descriptor, selections, initial_values_i)), selection_waslist))
 
+    def get_bin_count(self):
+        return reduce(lambda prev, binner: len(binner) * prev, self.binners, 1)
+
+    def memory_usage(self):
+        return self.nbytes
+
     def ideal_splits(self, nthreads):
         # We need to do some proper work on this, but this should already improve performance
         # Since if we have a lot of data per task, we should split up the work less
@@ -498,7 +528,7 @@ class TaskPartAggregation(TaskPart):
         logger.info(f'Estimate for ideal number of splits: {splits:,}')
         return max(min(nthreads, 2), splits)
 
-    def process(self, thread_index, i1, i2, filter_mask, *blocks):
+    def process(self, thread_index, i1, i2, filter_mask, selection_masks, blocks):
         # self.check()
         grid = self.grid
 
@@ -539,20 +569,24 @@ class TaskPartAggregation(TaskPart):
                 binner.clear_data_mask()
                 references.extend([block])
         all_aggregators = []
+
+        selection_index_global = 0
         for agg_desc, selections, aggregation2d, selection_waslist in self.aggregations:
             for selection_index, selection in enumerate(selections):
                 agg = aggregation2d[selection_index]
                 all_aggregators.append(agg)
                 selection_mask = None
                 if not (selection is None or selection is False):
-                    selection_mask = self.df.evaluate_selection_mask(selection, i1=i1, i2=i2, cache=True)  # TODO
+                    selection_mask = selection_masks[selection_index_global] # self.df.evaluate_selection_mask(selection, i1=i1, i2=i2, cache=True)  # TODO
                     # TODO: we probably want a way to avoid a to numpy conversion?
+                    assert selection_mask is not None
                     selection_mask = np.asarray(selection_mask)
                     references.append(selection_mask)
                     # some aggregators make a distiction between missing value and no value
                     # like nunique, they need to know if they should take the value into account or not
                     if hasattr(agg, 'set_selection_mask'):
                         agg.set_selection_mask(selection_mask)
+                selection_index_global += 1
                 if agg_desc.expressions:
                     assert len(agg_desc.expressions) in [1, 2], "only length 1 or 2 supported for now"
                     for i, expression in enumerate(agg_desc.expressions):
