@@ -9,6 +9,7 @@ import pkg_resources
 import numpy as np
 
 import vaex.promise
+import vaex.settings
 import vaex.encoding
 from .utils import _expand_shape
 from .datatype import DataType
@@ -19,7 +20,6 @@ register = vaex.encoding.make_class_registery('task')
 
 _lock = threading.Lock()
 _task_checker_types = {}
-task_checkers_type = os.environ.get("VAEX_TASK_CHECKER", "")
 
 
 class Checker:
@@ -39,6 +39,7 @@ def create_checkers():
                 for entry in pkg_resources.iter_entry_points(group="vaex.task.checker"):
                     _task_checker_types[entry.name] = entry.load()
     names = list(_task_checker_types.keys())
+    task_checkers_type = vaex.settings.main.task_tracker.type
     types = [k for k in task_checkers_type.split(",") if k]
     checkers = []
     for type in types:
@@ -68,12 +69,14 @@ class Task(vaex.promise.Promise):
     requires_fingerprint = False
     cacheable = True
     see_all = False
+    _toreject: Exception = None
 
     def __init__(self, df=None, expressions=[], pre_filter=False, name="task"):
         vaex.promise.Promise.__init__(self)
         self.df = df
         self.expressions = expressions
         self.expressions_all = list(expressions)
+        self.signal_start = vaex.events.Signal("start of task")
         self.signal_progress = vaex.events.Signal("progress (float)")
         self.progress_fraction = 0
         self.signal_progress.connect(self._set_progress)
@@ -98,6 +101,11 @@ class Task(vaex.promise.Promise):
             self._fingerprint = f'task-{self.name}-{task_fp}-{df_fp}'
         return self._fingerprint
 
+    def progress(self, fraction):
+        if not self.cancelled:
+            self.cancelled = not all(self.signal_progress.emit(fraction))
+        return not self.cancelled
+
     def _set_progress(self, fraction):
         self.progress_fraction = fraction
         return not self.cancelled  # don't cancel
@@ -116,7 +124,7 @@ class Task(vaex.promise.Promise):
 
     def create_next(self):
         ret = Task(self.df, [])
-        self.signal_progress.connect(ret.signal_progress.emit)
+        self.signal_progress.connect(lambda f: all(ret.signal_progress.emit(f)))
         return ret
 
     # def __repr__(self):
