@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import
+import ast
 import collections
 import concurrent.futures
 import contextlib
@@ -10,14 +11,17 @@ import os
 import platform
 import re
 import sys
+import threading
 import time
+from typing import MutableMapping
 import warnings
 import numbers
 import keyword
+import pkg_resources
 
+import dask.utils
 import numpy as np
 import pyarrow as pa
-import progressbar
 import six
 import yaml
 
@@ -34,6 +38,9 @@ osname = dict(darwin="osx", linux="linux", windows="windows")[platform.system().
 devmode = os.environ.get('VAEX_DEV', '0') == '1'
 
 
+# so that vaex can be imported without compiling the extenstions (like in readthedocs)
+has_c_extension = os.environ.get('VAEX_NO_C_EXTENSIONS', None) != "1"
+
 class AttrDict(dict):
     def __init__(self, *args, **kwargs):
         dict.__init__(self, *args, **kwargs)
@@ -45,6 +52,40 @@ class AttrDict(dict):
         self[key] = value
 
 
+class RegistryCallable(MutableMapping):
+    '''lazily load entries from entry_points'''
+    def __init__(self, entry_points : str, typename : str):
+        self.entry_points = entry_points
+        self.typename = typename
+        self.registry = {}
+        self.lock = threading.Lock()
+
+    def __getitem__(self, name):
+        if name in self.registry:
+            return self.registry[name]
+        with self.lock:
+            if name in self.registry:
+                return self.registry[name]
+            for entry in pkg_resources.iter_entry_points(group=self.entry_points):
+                self.registry[entry.name] = entry.load()
+
+        if name not in self.registry:
+            raise NameError(f'No {self.typename} registered with name {name!r} under entry_point {self.entry_points!r}')
+        return self.registry[name]
+
+    def __delitem__(self, name):
+        del self.registry[name]
+
+    def __iter__(self):
+        return self.registry.__iter__()
+
+    def __len__(self):
+        return self.registry.__len__()
+
+    def __setitem__(self, name, value):
+        self.registry.__setitem__(name, value)
+
+
 def deprecated(reason):
     def wraps(f):
         if not f.__doc__:
@@ -81,6 +122,8 @@ def subdivide_mask(mask, parts=None, max_length=None, logical_length=None):
     """
     if logical_length is None:
         logical_length = np.asarray(mask).sum()
+    if logical_length == 0:
+        return
     raw_length = len(np.asarray(mask))
     if max_length is None:
         max_length = (logical_length + parts - 1) / parts
@@ -199,11 +242,26 @@ class Timer(object):
         return False
 
 
+def get_vaex_home():
+    '''Get vaex home directory, defaults to $HOME/.vaex.
+
+    The $VAEX_PATH_HOME environment variable can be set to override this default.
+
+    If both $VAEX_PATH_HOME and $HOME are not define, the current working directory is used.
+    '''
+    if 'VAEX_PATH_HOME' in os.environ:
+        return os.environ['VAEX_PATH_HOME']
+    elif 'HOME' in os.environ:
+        return os.path.join(os.environ['HOME'], ".vaex")
+    else:
+        return os.getcwd()
+
+
 def get_private_dir(subdir=None, *extra):
-    path = os.path.expanduser('~/.vaex')
+    path = get_vaex_home()
     if subdir:
         path = os.path.join(path, subdir, *extra)
-    os.makedirs(path,exist_ok=True)
+    os.makedirs(path, exist_ok=True)
     return path
 
 
@@ -214,191 +272,6 @@ def make_list(sequence):
         return list(sequence)
 
 
-# from progressbar import AnimatedMarker, Bar, BouncingBar, Counter, ETA, \
-# FileTransferSpeed, FormatLabel, Percentage, \
-# ProgressBar, ReverseBar, RotatingMarker, \
-# SimpleProgress, Timer, AdaptiveETA, AbsoluteETA, AdaptiveTransferSpeed
-# from progressbar.widgets import TimeSensitiveWidgetBase, FormatWidgetMixin
-
-
-class CpuUsage(progressbar.widgets.FormatWidgetMixin, progressbar.widgets.TimeSensitiveWidgetBase):
-    def __init__(self, format='CPU Usage: %(cpu_usage)s%%', usage_format="% 5d"):
-        super(CpuUsage, self).__init__(format=format)
-        self.usage_format = usage_format
-        self.utime_0 = None
-        self.stime_0 = None
-        self.walltime_0 = None
-
-    def __call__(self, progress, data):
-        utime, stime, child_utime, child_stime, walltime = os.times()
-        if self.utime_0 is None:
-            self.utime_0 = utime
-        if self.stime_0 is None:
-            self.stime_0 = stime
-        if self.walltime_0 is None:
-            self.walltime_0 = walltime
-        data["utime_0"] = self.utime_0
-        data["stime_0"] = self.stime_0
-        data["walltime_0"] = self.walltime_0
-
-        delta_time = utime - self.utime_0 + stime - self.stime_0
-        delta_walltime = walltime - self.walltime_0
-        # print delta_time, delta_walltime, utime, self.utime_0, stime, self.stime_0
-        if delta_walltime == 0:
-            data["cpu_usage"] = "---"
-        else:
-            cpu_usage = delta_time / (delta_walltime * 1.) * 100
-            data["cpu_usage"] = self.usage_format % cpu_usage
-        # utime0, stime0, child_utime0, child_stime0, walltime0 = os.times()
-        return progressbar_mod.widgets.FormatWidgetMixin.__call__(self, progress, data)
-
-
-progressbar_mod = progressbar
-
-
-def _progressbar_progressbar2(type=None, name="processing", max_value=1):
-    widgets = [
-        name,
-        ': ', progressbar_mod.widgets.Percentage(),
-        ' ', progressbar_mod.widgets.Bar(),
-        ' ', progressbar_mod.widgets.ETA(),
-        # ' ', progressbar_mod.widgets.AdaptiveETA(),
-        ' ', CpuUsage()
-    ]
-    bar = progressbar_mod.ProgressBar(widgets=widgets, max_value=max_value)
-    bar.start()
-    return bar
-    # FormatLabel('Processed: %(value)d lines (in: %(elapsed)s)')
-
-
-def _progressbar_vaex(type=None, name="processing", max_value=1):
-    import vaex.misc.progressbar as pb
-    return pb.ProgressBar(0, 1)
-
-def _progressbar_widget(type=None, name="processing", max_value=1):
-    import vaex.misc.progressbar as pb
-    return pb.ProgressBarWidget(0, 1, name=name)
-
-
-_progressbar_typemap = {}
-_progressbar_typemap['progressbar2'] = _progressbar_progressbar2
-_progressbar_typemap['vaex'] = _progressbar_vaex
-_progressbar_typemap['widget'] = _progressbar_widget
-
-
-def progressbar(type_name=None, title="processing", max_value=1):
-    type_name = type_name or 'vaex'
-    return _progressbar_typemap[type_name](name=title)
-
-
-def progressbar_widget():
-    pass
-
-
-class _progressbar(object):
-    pass
-
-
-class _progressbar_wrapper(_progressbar):
-    def __init__(self, bar):
-        self.bar = bar
-
-    def __call__(self, fraction):
-        self.bar.update(fraction)
-        if fraction == 1:
-            self.bar.finish()
-        return True
-
-    def status(self, name):
-        self.bar.bla = name
-
-
-class _progressbar_wrapper_sum(_progressbar):
-    def __init__(self, children=None, next=None, bar=None, parent=None, name=None):
-        self.next = next
-        self.children = children or list()
-        self.finished = False
-        self.last_fraction = None
-        self.fraction = 0
-        self.bar = bar
-        self.parent = parent
-        self.name = name
-        self.cancelled = False
-        self.oncancel = lambda: None
-
-    def cancel(self):
-        self.cancelled = True
-
-    def __repr__(self):
-        name = self.__class__.__module__ + "." + self.__class__.__name__
-        return "<%s(name=%r)> instance at 0x%x" % (name, self.name, id(self))
-
-    def add(self, name=None):
-        pb = _progressbar_wrapper_sum(parent=self, name=name)
-        self.children.append(pb)
-        return pb
-
-    def add_task(self, task, name=None):
-        pb = self.add(name)
-        pb.oncancel = task.cancel
-        task.signal_progress.connect(pb)
-        if self.bar and hasattr(self.bar, 'add_child'):
-            self.bar.add_child(pb, task, name)
-
-    def __call__(self, fraction):
-        if self.cancelled:
-            return False
-        # ignore fraction
-        result = True
-        if len(self.children) == 0:
-            self.fraction = fraction
-        else:
-            self.fraction = sum([c.fraction for c in self.children]) / len(self.children)
-        fraction = self.fraction
-        if fraction != self.last_fraction:  # avoid too many calls
-            if fraction == 1 and not self.finished:  # make sure we call finish only once
-                self.finished = True
-                if self.bar:
-                    self.bar.finish()
-            elif fraction != 1:
-                if self.bar:
-                    self.bar.update(fraction)
-            if self.next:
-                result = self.next(fraction)
-        if self.parent:
-            assert self in self.parent.children
-            result = self.parent(None) in [None, True] and result  # fraction is not used anyway..
-            if result is False:
-                self.oncancel()
-        self.last_fraction = fraction
-        return result
-
-    def status(self, name):
-        pass
-
-
-def progressbars(f=True, next=None, name=None):
-    if isinstance(f, _progressbar_wrapper_sum):
-        return f
-    if callable(f):
-        next = f
-        f = False
-    if f in [None, False]:
-        return _progressbar_wrapper_sum(next=next, name=name)
-    else:
-        if f is True:
-            return _progressbar_wrapper_sum(bar=progressbar(), next=next, name=name)
-        elif isinstance(f, six.string_types):
-            return _progressbar_wrapper_sum(bar=progressbar(f), next=next, name=name)
-        else:
-            return _progressbar_wrapper_sum(next=next, name=name)
-
-
-def progressbar_callable(title="processing", max_value=1):
-    bar = progressbar(title=title, max_value=max_value)
-    return _progressbar_wrapper(bar)
-
-
 def confirm_on_console(topic, msg):
     done = False
     print(topic)
@@ -411,7 +284,7 @@ def confirm_on_console(topic, msg):
 
 
 def yaml_dump(f, data):
-    yaml.safe_dump(data, f, default_flow_style=False, encoding='utf-8', allow_unicode=True)
+    yaml.safe_dump(data, f, default_flow_style=False, encoding='utf-8', allow_unicode=True, sort_keys=False)
 
 
 def yaml_load(f):
@@ -452,23 +325,6 @@ def read_json_or_yaml(file, fs_options={}, fs=None, old_style=True):
         file.close()
 
 
-# from http://stackoverflow.com/questions/5121931/in-python-how-can-you-load-yaml-mappings-as-ordereddicts
-
-_mapping_tag = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG
-
-
-def dict_representer(dumper, data):
-    return dumper.represent_dict(data.iteritems() if hasattr(data, "iteritems") else data.items())
-
-
-def dict_constructor(loader, node):
-    return collections.OrderedDict(loader.construct_pairs(node))
-
-
-yaml.add_representer(collections.OrderedDict, dict_representer, yaml.SafeDumper)
-yaml.add_constructor(_mapping_tag, dict_constructor, yaml.SafeLoader)
-
-
 def check_memory_usage(bytes_needed, confirm):
     psutil = optional_import('psutil')
     if bytes_needed > psutil.virtual_memory().available:
@@ -674,6 +530,13 @@ def _normalize_selection_name(name):
         return name
 
 
+def _normalize_selection(selection):
+    if isinstance(selection, (list, tuple)):
+        return type(selection)([_normalize_selection_name(k) for k in selection])
+    else:
+        return _normalize_selection_name(selection)
+
+
 def _parse_n(n):
     if isinstance(n, six.string_types):
         if n == "normalize":
@@ -770,7 +633,7 @@ def _ensure_string_from_expression(expression):
     elif isinstance(expression, six.string_types):
         return expression
     elif isinstance(expression, vaex.expression.Expression):
-        return expression._label
+        return expression.expression
     else:
         raise ValueError('%r is not of string or Expression type, but %r' % (expression, type(expression)))
 
@@ -1028,6 +891,20 @@ def div_ceil(n, d):
     return (n + d - 1) // d
 
 
+def get_env_memory(key, default=None):
+    value = os.environ.get(key, default)
+    if value is not None:
+        try:
+            value = ast.literal_eval(value)
+        except:
+            pass
+        if isinstance(value, str):
+            value = dask.utils.parse_bytes(value)
+        if not isinstance(value, int):
+            raise TypeError(f"Expected env var {key} to be of integer type")
+    return value
+
+
 def get_env_type(type, key, default=None):
     '''Get an env var named key, and cast to type
 
@@ -1052,7 +929,6 @@ def get_env_type(type, key, default=None):
         # support empty strings
         value = default
     if value is not None:
-        import ast
         return type(ast.literal_eval(repr(value)))
 
 
@@ -1063,3 +939,9 @@ def dropnan(sequence, expect=None):
     if expect is not None:
         assert len(sequence) - len(non_nan) == 1, "expected 1 nan value"
     return original_type(non_nan)
+
+
+# backwards compatibility
+def progressbars(*args, **kwargs):
+    from .progress import tree
+    return tree(*args, **kwargs)
